#!/usr/bin/python

import time, sys, json
import paho.mqtt.client as mqtt

def setWallboxPower(client, gridPower):
	global config

	currentSetpoint = gridPower * config['netzero']['powerkp'] + setWallboxPower.errSum * config['netzero']['powerki']				
	currentSetpoint = min(16, max(0, currentSetpoint))

	if currentSetpoint < 16 and currentSetpoint >= 0:
		setWallboxPower.errSum = setWallboxPower.errSum + gridPower
								
	client.publish("/charger/setpoint/current", currentSetpoint)		

def calcGridPower(client):
	global lastGridPower
	global lastConsumePower
	global lastSupplyPower

	if abs(lastConsumePower) > abs(lastSupplyPower):
		lastGridPower = (lastConsumePower + lastGridPower) / 2
	else:
		lastGridPower = (-lastSupplyPower + lastGridPower) / 2
		
	client.publish("/netzero/gridpower", lastGridPower)

def on_message(client, userdata, msg):
	global config
	global lastGridPower
	global lastConsumePower
	global lastSupplyPower

	if msg.topic == config['meter']['consume_topic']:
		lastConsumePower = float(msg.payload)
		calcGridPower(client)
		setWallboxPower(client, lastGridPower)
	elif msg.topic == config['meter']['supply_topic']:
		lastSupplyPower = float(msg.payload)
		calcGridPower(client)
		setWallboxPower(client, lastGridPower)
	elif msg.topic == "/charger/info/power":
		chargerPower = float(msg.payload)
		lastChargerPower = round((chargerPower + lastChargerPower) / 2, 2) #IIR filter
		publishBatteryPower(client)

with open("config.json") as configFile:
	config = json.load(configFile)
	
client = mqtt.Client("netZeroController")
client.on_message = on_message
client.connect("localhost", 1883, 60)
client.subscribe(config['meter']['consume_topic'])
client.subscribe(config['meter']['supply_topic'])
client.subscribe("/charger/info/maxpower")
client.subscribe("/charger/info/power")

lastChargerPower = 0
lastGridPower = 0
lastConsumePower = 0
lastSupplyPower = 0

setWallboxPower.errSum = 0

client.loop_forever()

